import { Action } from 'redux';
import { effects, Effect } from 'redux-saga';
import { GlobalState } from './containerModule';
export interface DefaultReducer<State> {
    (state: State, action: Action<any>): State;
}
export interface SagaEnhancer {
    emit: Func;
    take: Func;
    syncPut: Func;
}
export interface SagaEffects {
    take: typeof effects.take;
    put: typeof effects.put;
    all: typeof effects.all;
    race: typeof effects.race;
    call: typeof effects.call;
    apply: typeof effects.apply;
    cps: typeof effects.cps;
    fork: typeof effects.fork;
    spawn: typeof effects.spawn;
    join: typeof effects.join;
    cancel: typeof effects.cancel;
    select: typeof effects.select;
    actionChannel: typeof effects.actionChannel;
    flush: typeof effects.flush;
    getContext: typeof effects.getContext;
    setContext: typeof effects.setContext;
    takeEvery: typeof effects.takeEvery;
    takeLatest: typeof effects.takeLatest;
    throttle: typeof effects.throttle;
}
export interface Func {
    (...args: any[]): any;
}
export interface SelectorsOf<State> {
    [key: string]: (state: State, ...args: any[]) => any;
}
export interface ReducerOf<State> {
    [key: string]: (state: State, action: {
        type: string;
        payload: any;
    }) => State;
}
export interface ActionsOf<ActionCreators, Reducers, Effects, Selectors> {
    <S extends KopSelectors<Selectors>>(param: {
        actions: any;
        selectors: S;
        dispatch: Func;
        createAction: (type: string) => Action;
    }): ActionCreators;
}
export interface EffectsOf<Selectors, Reducers, Effects> {
    <S extends KopSelectors<Selectors>>(param: {
        actions: any;
        selectors: S;
        sagaEffects: SagaEffects;
        enhancer: SagaEnhancer;
    }): Effects;
}
export declare type ReducerToAction<Reducer> = Reducer extends (state: any, action: infer A) => any ? A extends {
    payload: infer P;
} ? (payload: P) => any : () => any : never;
export declare type EffectToAction<Effect> = Effect extends () => any ? () => any : Effect extends (payload: {
    payload: infer P;
}) => any ? (payload: P) => any : never;
export declare type KopActions<Reducers, Effects, ActionCreators = {}> = {
    [K in keyof Reducers]: ReducerToAction<Reducers[K]>;
} & {
    [K in keyof Effects]: EffectToAction<Effects[K]>;
} & ActionCreators;
export interface CommonSelectors {
    getModuleState: Func;
    getGlobalState: Func;
}
export declare type KopSelectors<T> = {
    [P in keyof T]: T[P] extends (state: any) => infer A ? (state: GlobalState) => A : T[P] extends (state: any, payload: {
        payload: infer B;
    }) => infer A ? (state: GlobalState, payload: B) => A : T[P] extends (state: any, payload: any) => infer A ? (state: GlobalState, payload: any) => A : (state: GlobalState) => any;
} & CommonSelectors;
export interface KopModule<Selectors, Reducers, Effects> {
    selectors?: KopSelectors<Selectors>;
    actions?: KopActions<Reducers, Effects>;
    event?: (evt: string) => string;
    setup?: Func;
    reset?: Func;
}
export interface WrappedAction {
    type: string;
    __namespace: string;
    __actionKind: symbol;
    __type: string;
    __kopFunction: Function;
    __changeLoading: Function;
    __moduleType: symbol;
    toString: Function;
}
export interface BaseAction {
    type: string;
    payload: any;
}
export interface BaseSelectors<State> {
    [key: string]: (state: State, ...args: any[]) => any;
}
export interface BaseReducers<State> {
    [key: string]: (state: State, action: BaseAction) => State;
}
export interface Enhancer {
    syncPut: Effect;
    emit: Function;
}
export interface BaseEffects<Selector> {
    (actions: any, selectors: Selector, sagaEffects: any, enhancer: Enhancer): {
        [key: string]: Effect;
    };
}
export interface BaseModuleOption<State> {
    initialState: State;
    selectors: BaseSelectors<State>;
    reducers: BaseReducers<State>;
    effects: BaseEffects<BaseSelectors<State>>;
}
export interface BaseModuleActions {
    [index: string]: (...param: any) => any;
}
export interface BaseModuleSelectors {
    [index: string]: (...param: any) => any;
}
export interface BaseReducer {
    (state: object): any;
}
export interface BaseModule {
    namespace: string;
    type: symbol;
    parent: object;
    presenter: {
        loaded: boolean;
    };
    _initialState?: {
        '@@loading': boolean;
        [key: string]: any;
    };
    selectors?: BaseModuleSelectors;
    _reducers?: BaseReducer;
    event?: any;
    actions?: BaseModuleActions;
    setup?: Func;
    reset?: Func;
    lifecycle?: Func;
}
